/* Shared library add-on to iptables to add RATE support.
 *
 * Jerome de Vivie   <devivie@info.enserb.u-bordeaux.fr>
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include "iptables.h"
#include "ip_tables.h"
#include "ipt_RATE.h"

/* Function which prints out usage message. */
static void 
help(void)
{
	printf(
"RATE options:\n"
"--rate avg			max average time allowed between 2 pkts (ms)\n"
"--rate-peak peak		max time between 2 pkts, (peak < avg) (ms)\n"
"--rate-no-log			Suppress kernel flood reports\n\n");
}

static struct option opts[] = {
	{ "rate", 1, 0, '%' },
	{ "rate-peak", 1, 0, '$' },
	{ "rate-no-log", 0, 0, '?' },
	{ 0 }
};

static void
init(union ipt_targinfo *targinfo)
{
	/* default initialisation */
	struct ipt_rateinfo *r = (struct ipt_rateinfo *)targinfo->other;
	r->avg = IPT_RATE_AVG;
	r->peak = IPT_RATE_PEAK;
	r->log = 1;
}

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      struct ipt_entry *entry)
{	
	struct ipt_rateinfo *r = (struct ipt_rateinfo *)entry->targinfo.other;
	
	switch(c) {
	case '%':
		if (check_inverse(optarg, &invert))
			exit_error(PARAMETER_PROBLEM, 
				   "Unexpected `!' after --rate");
		if(sscanf(optarg,"%hu",&r->avg) != 1)
			exit_error(PARAMETER_PROBLEM, "rate `%s' ambiguous",optarg);
		break;	

	case '$':			
		if (check_inverse(optarg, &invert))
			exit_error(PARAMETER_PROBLEM, 
				   "Unexpected `!' after --rate-peak");

		if(sscanf(optarg,"%hu",&r->peak) != 1)
			exit_error(PARAMETER_PROBLEM, "rate-peak `%s' ambiguous",optarg);
		break;	
	case '?':
		if (invert)
			exit_error(PARAMETER_PROBLEM,
				   "Unexpected `!' before --rate-no-log");
		r->log = 0;
		break;
	default:
		return 0;
	}
	return 1;
}	

/* Final check; nothing. */
static void final_check(unsigned int flags)
{
}

/* Prints out the union ipt_targinfo. */
static void print(const union ipt_targinfo *info, int numeric)
{
	struct ipt_rateinfo *r = (struct ipt_rateinfo *)info->other;
	printf("avg %u, peak %u ",r->avg,r->peak);
}

struct iptables_target rate
= { NULL,
    "RATE",
    &help,
    &init,
    &parse,
    &final_check,
    &print,
    opts
};

void _init(void)
{
	register_target(&rate);
}
